" Vim syntax file
" Language:           ConTeXt typesetting engine
" Maintainer:         Nicola Vitacolonna <nvitacolonna@gmail.com>
" Former Maintainers: Nikolai Weibull <now@bitwi.se>
" Latest Revision:    2016 Oct 10

if exists("b:current_syntax")
  finish
endif

runtime! syntax/plaintex.vim
unlet b:current_syntax

let s:cpo_save = &cpo
set cpo&vim

" Dictionary of (filetype, group) pairs to highlight between \startGROUP \stopGROUP.
let s:context_include = get(b:, 'context_include',
      \ get(g:, 'context_include', {'javascript': 'JS', 'xml': 'XML'}))

" For backward compatibility (g:context_include used to be a List)
if type(s:context_include) ==# v:t_list
  let g:context_metapost = (index(s:context_include, 'mp') != -1)
  let s:context_include = filter(
        \ {'c': 'C', 'javascript': 'JS', 'ruby': 'Ruby', 'xml': 'XML'},
        \ { k,_ -> index(s:context_include, k) != -1 }
        \ )
endif

syn spell   toplevel

syn match   contextBlockDelim display '\\\%(start\|stop\)\a\+'
                              \ contains=@NoSpell

syn region  contextEscaped    display matchgroup=contextPreProc
                              \ start='\\type\z(\A\)' end='\z1'
syn region  contextEscaped    display matchgroup=contextPreProc
                              \ start='\\type\={' end='}'
syn region  contextEscaped    display matchgroup=contextPreProc
                              \ start='\\type\=<<' end='>>'
syn region  contextEscaped    matchgroup=contextPreProc
                              \ start='\\start\z(\a*\%(typing\|typen\)\)'
                              \ end='\\stop\z1' contains=plaintexComment keepend
syn region  contextEscaped    display matchgroup=contextPreProc
                              \ start='\\\h\+Type{' end='}'
syn region  contextEscaped    display matchgroup=contextPreProc
                              \ start='\\Typed\h\+{' end='}'

syn match   contextBuiltin    display contains=@NoSpell
      \ '\\\%(unprotect\|protect\|unexpanded\)'

syn match   contextPreProc    '^\s*\\\%(start\|stop\)\=\%(component\|environment\|project\|product\).*$'
                              \ contains=@NoSpell

if get(b:, 'context_metapost', get(g:, 'context_metapost', 1))
  syn include @mpTop          syntax/mp.vim
  unlet b:current_syntax

  syn region  contextMPGraphic  transparent matchgroup=contextBlockDelim
                                \ start='\\start\z(\a*MPgraphic\|MP\%(code\|page\|inclusions\|run\)\).*'
                                \ end='\\stop\z1'
                                \ contains=@mpTop
endif

" TODO: support \directlua and \ctxlua
if get(b:, 'context_lua', get(g:, 'context_lua', 1))
  syn include @luaTop          syntax/lua.vim
  unlet b:current_syntax

  syn region  contextLuaCode    transparent matchgroup=contextBlockDelim
                                \ start='\\startluacode'
                                \ end='\\stopluacode'
                                \ contains=@luaTop
endif

" TODO: also need to implement this for \\typeC or something along those
" lines.
for synname in keys(s:context_include)
  execute 'syn include @' . synname . 'Top' 'syntax/' . synname . '.vim'
  unlet b:current_syntax
  execute 'syn region context' . s:context_include[synname] . 'Code'
        \ 'transparent matchgroup=contextBlockDelim'
        \ 'start=+\\start' . s:context_include[synname] . '+'
        \ 'end=+\\stop' . s:context_include[synname] . '+'
        \ 'contains=@' . synname . 'Top'
endfor

syn match   contextSectioning '\\chapter\>' contains=@NoSpell
syn match   contextSectioning '\\\%(sub\)*section\>' contains=@NoSpell

syn match   contextSpecial    '\\crlf\>\|\\par\>\|-\{2,3}\||[<>/]\=|'
                              \ contains=@NoSpell
syn match   contextSpecial    /\\[`'"]/
syn match   contextSpecial    +\\char\%(\d\{1,3}\|'\o\{1,3}\|"\x\{1,2}\)\>+
                              \ contains=@NoSpell
syn match   contextSpecial    '\^\^.'
syn match   contextSpecial    '`\%(\\.\|\^\^.\|.\)'

syn match   contextStyle      '\\\%(em\|ss\|hw\|cg\|mf\)\>'
                              \ contains=@NoSpell
syn match   contextFont       '\\\%(CAP\|Cap\|cap\|Caps\|kap\|nocap\)\>'
                              \ contains=@NoSpell
syn match   contextFont       '\\\%(Word\|WORD\|Words\|WORDS\)\>'
                              \ contains=@NoSpell
syn match   contextFont       '\\\%(vi\{1,3}\|ix\|xi\{0,2}\)\>'
                              \ contains=@NoSpell
syn match   contextFont       '\\\%(tf\|b[si]\|s[cl]\|os\)\%(xx\|[xabcd]\)\=\>'
                              \ contains=@NoSpell

hi def link contextBlockDelim Keyword
hi def link contextBuiltin    Keyword
hi def link contextDelimiter  Delimiter
hi def link contextPreProc    PreProc
hi def link contextSectioning PreProc
hi def link contextSpecial    Special
hi def link contextType       Type
hi def link contextStyle      contextType
hi def link contextFont       contextType

let b:current_syntax = "context"

let &cpo = s:cpo_save
unlet s:cpo_save
